/**
 * contains utility functions for work with strings
 * @type {{packagePath: (function(*): string), dashName: (function(*): string), path: (function(*): (string|*)),
 * underscoreName: (function(*): string), nameFromDbScheme: (function(*): string), nameLower: (function(*)),
 * camelCase: (function(*): string)}}
 */
module.exports = {
    /**
     * makes first letter of string lower case
     * @param name
     * @returns {string}
     */
    nameLower: function (name) {
        return name.charAt(0).toLocaleLowerCase() + name.slice(1);
    },

    /**
     * deletes _ from string
     * @param name - inputed string
     * @returns {string}
     */
    nameFromDbScheme: function (name) {
        return name.split('_').join('');
    },

    /**
     * creates path from package name
     * @param packageName string
     * @returns {string}
     */
    packagePath: function (packageName) {
        return packageName.split('.').join('/');
    },

    /**
     * replace spaces in name with dashes
     * @param name string parameter
     * @returns {string}
     */
    dashName: function (name) {
        return name.toLocaleLowerCase().split(' ').join('-');
    },

    /**
     * replace spaces in name with underscore
     * @param name string parameter
     * @returns {string}
     */
    underscoreName: function (name) {
        return name.toLocaleLowerCase().split(' ').join('_');
    },

    /**
     * deletes spaces from string a makes first letter of each word uppercase
     * @param name string that is edited
     * @returns {string}
     */
    camelCase: function (name) {
        let words = name.split(' ');
        for (let i = 0; i < words.length; i++) {
            words[i] = words[i].charAt(0).toLocaleUpperCase() + words[i].slice(1);
        }
        return words.join('');
    },

    /**
     * adds / to end of path if not present
     * if not return empty string
     * @param path - path we are editing
     * @returns {string|*}
     */
    path: function (path) {
        if (path != null && path.length > 0) {
            if (path.charAt(path.length - 1) !== '/') {
                return path.concat('/');
            }
        }
        return "";
    }

}

