'use strict';
const Generator = require('yeoman-generator');

let outputFile;
let columns;
let gridName;
let editable;

module.exports = class extends Generator {
    constructor(args, opts) {
        super(args, opts);

        this.argument("outputFilename", {
            type: String,
            required: false,
            description: "Filename to where is the grid generated",
            default: "grid.html"
        });

        outputFile = opts.outputFilename;
    }

    /**
     * reads user inputs
     */
    async prompting() {
        this.answers = await this.prompt([
            //definition of common inputs
            {
                type: "confirm",
                name: "editable",
                message: "Is the grid editable ? ",
            },
            {
                type: "input",
                name: "gridName",
                message: "Grid name, format (camelCase)"
            },
            {
                type: "input",
                name: "number",
                message: "Number of colums",
            },
        ]);
        let count = this.answers.number;
        columns = [count];
        editable = this.answers.editable;
        gridName = this.answers.gridName.charAt(0).toLocaleUpperCase() + this.answers.gridName.slice(1);

        if (editable) {
            for (let i = 0; i < count; i++) {
                columns[i] = await this.prompt([
                    //definition of column inputs
                    {
                        type: "input",
                        name: "name",
                        message: (i + 1) + ". column name",
                    },
                    {
                        type: "list",
                        name: "type",
                        message: (i + 1) + ". column type",
                        choices: ['text', 'date', 'numeric', 'boolean'],
                    },
                    {
                        type: 'confirm',
                        name: 'editable',
                        message: (i + 1) + ". column is editable"
                    }
                ]);
            }
        } else {
            for (let i = 0; i < count; i++) {
                columns[i] = await this.prompt([
                    //definition of column inputs
                    {
                        type: "input",
                        name: "name",
                        message: (i + 1) + ". name",
                    },
                    {
                        type: "list",
                        name: "type",
                        message: (i + 1) + ". type",
                        choices: ['text', 'date', 'numeric', 'boolean'],
                    },
                ]);
            }
        }
    }

    async writing() {
        if (editable) {
            //generating editable grid
            this.fs.copyTpl(
                this.templatePath('editableGrid.html'),
                this.destinationPath(outputFile), {
                    columns: columns,
                    gridName: gridName,
                }
            );
        } else {
            //generating non-editabel grid
            this.fs.copyTpl(
                this.templatePath('grid.html'),
                this.destinationPath(outputFile), {
                    columns: columns,
                    gridName: gridName,
                }
            );
        }
    }
}
