'use strict';

const Generator = require('yeoman-generator');
//contains util function form strings
const stringUtil = require('./string-utils');
let path;

module.exports = class extends Generator {

    constructor(args, opts) {
        // Calling the super constructor is important so our generator is correctly set up
        super(args, opts);
        this.argument("projectPath", {
            type: String,
            required: false,
            description: "Path to project folder",
            default: ""
        });

        path = stringUtil.folderPath(opts.projectPath);
    }

    /**
     * reads user inputs
     */
    async prompting() {
        this.answers = await this.prompt([
            {
                type: "input",
                name: "componentName",
                message: "Component name, format (example-name)",
            },
            {
                type: "input",
                name: "rootPackageName",
                message: "root package name",
            },
        ]);
    }

    /**
     * generete files from templates
     */
    async writing() {
        //declaration of all constants used in templates
        //package name seperated with dots
        const rootPackageName = this.answers.rootPackageName;
        //package name seperated with slashes
        const rootPackagePath = stringUtil.packagePath(rootPackageName);
        //package name with cut last package
        const rootPackageCut = stringUtil.cutLastPackage(rootPackageName);
        //component name separated with dashes
        const componentName = this.answers.componentName;
        //component name seperated with dashes all lower case
        const componentNameLowerDash = componentName.toLocaleLowerCase();
        //component name in camel case first letter Upper case
        const componentNameCamel = stringUtil.camelCase(componentName);
        //conmponent name in camel case first letter Lower case
        const componentNameCamelLower = stringUtil.nameLower(componentNameCamel);
        //component name seperated with dots lower case
        const componentNameAllLowerDot = stringUtil.dotSeperator(componentName).toLocaleLowerCase();
        // component name seperated with spaces
        const componentNameSpace = stringUtil.spaceSeperator(componentName);
        //component name seperated with spaces all lower
        const componentNameAllLowerSpace = stringUtil.spaceSeperator(componentName).toLocaleLowerCase();

        //project root

        this.fs.copyTpl(
            this.templatePath('pom.xml'),
            this.destinationPath(path + 'pom.xml'), {
                componentNameAllLowerDash: componentNameLowerDash,
                componentNameSpace: componentNameSpace,
                componentNameAllLowerSpace: componentNameAllLowerSpace,
                packageNameCut: rootPackageCut,
            }
        );

        this.fs.copyTpl(
            this.templatePath('.gitignore'),
            this.destinationPath(path + '.gitignore')
        );

        //root package

        //creates Application file, respsonsible for starting spring context
        this.fs.copyTpl(
            this.templatePath('java/Application.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/' + componentNameCamel + 'Application.java'), {
                rootPackageName: rootPackageName,
                componentName: componentNameCamel,
            }
        );

        //creates servlet initializer file
        this.fs.copyTpl(
            this.templatePath('java/ServletInitializer.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/ServletInitializer.java'), {
                rootPackageName: rootPackageName,
                componentName: componentNameCamel,
            }
        );

        //web package

        //creates web config file
        this.fs.copyTpl(
            this.templatePath('java/web/WebConfig.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/config/web/WebConfig.java'), {
                rootPackageName: rootPackageName,
            }
        );

        //swagger package

        //creates swagger config file
        this.fs.copyTpl(
            this.templatePath('java/swagger/SwaggerConfig.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/config/swagger/SwaggerConfig.java'), {
                rootPackageName: rootPackageName,
                componentNameSpace: componentNameSpace,
            }
        );

        //security package

        this.fs.copyTpl(
            this.templatePath('java/security/MethodSecurityConfig.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/config/security/MethodSecurityConfig.java'), {
                rootPackageName: rootPackageName,
            }
        );

        this.fs.copyTpl(
            this.templatePath('java/security/SecurityConfig.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/config/security/SecurityConfig.java'), {
                rootPackageName: rootPackageName,
            }
        );

        //persistence package

        //creates db config
        this.fs.copyTpl(
            this.templatePath('java/persistence/DbConfig.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/config/persistence/' + componentNameCamel + 'DbConfig.java'), {
                rootPackageName: rootPackageName,
                componentName: componentNameCamel,
                componentNameLower: componentNameCamelLower,
                componentNameAllLowerDash: componentNameLowerDash,
                componentNameAllLowerDot: componentNameAllLowerDot,
            }
        );

        //creates parametr config
        this.fs.copyTpl(
            this.templatePath('java/persistence/ParameterConfig.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/config/persistence/ParameterConfig.java'), {
                rootPackageName: rootPackageName,
                componentNameLower: componentNameCamelLower,
            }
        );

        //creates physical naming strategy file
        this.fs.copyTpl(
            this.templatePath('java/persistence/PhysicalNamingStrategy.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/config/persistence/' + componentNameCamel + 'PhysicalNamingStrategy.java'), {
                rootPackageName: rootPackageName,
                componentName: componentNameCamel
            }
        );

        //db package

        //creates parametr repository
        this.fs.copyTpl(
            this.templatePath('java/db/ParametrRepository.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/db/repo/konfigurace/ParametrRepository.java'), {
                rootPackageName: rootPackageName,
            }
        );

        //creates parametr group repository
        this.fs.copyTpl(
            this.templatePath('java/db/ParametrSkupinaRepository.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/db/repo/konfigurace/ParametrSkupinaRepository.java'), {
                rootPackageName: rootPackageName,
            }
        );

        //resources

        //creates OpenAPI generator ignore file
        this.fs.copyTpl(
            this.templatePath('resources/.openapi-generator-ignore'),
            this.destinationPath(path + 'src/main/resources/.openapi-generator-ignore')
        );

        //creates properties file
        this.fs.copyTpl(
            this.templatePath('resources/application.properties'),
            this.destinationPath(path + 'src/main/resources/application.properties'), {
                componentNameAllLowerDot: componentNameAllLowerDot,
            }
        );

        //creates logging file
        this.fs.copyTpl(
            this.templatePath('resources/log4j2.xml'),
            this.destinationPath(path + 'src/main/resources/log4j2.xml')
        );

        //creates message properties file
        this.fs.copyTpl(
            this.templatePath('resources/messages.properties'),
            this.destinationPath(path + 'src/main/resources/messages.properties')
        );

        //creates czech locale message properties file
        this.fs.copyTpl(
            this.templatePath('resources/messages_cs_CZ.properties'),
            this.destinationPath(path + 'src/main/resources/messages_cs_CZ.properties')
        );

        //creates english locale properties file
        this.fs.copyTpl(
            this.templatePath('resources/messages_en_US.properties'),
            this.destinationPath(path + 'src/main/resources/messages_en_US.properties')
        );

        //generates empty master changelog file
        this.fs.copyTpl(
            this.templatePath('resources/db.changelog-master.xml'),
            this.destinationPath(path + 'src/main/resources/changelog/' + componentNameLowerDash + '-db.changelog-master.xml')
        );

        //webapp

        //creates index page
        this.fs.copyTpl(
            this.templatePath('webapp/index.html'),
            this.destinationPath(path + 'src/main/webapp/index.html'), {
                componentNameAllLowerSpace: componentNameAllLowerSpace,
            }
        );

        //webapp WEB-INF

        //creates jboss deployment structure config file
        this.fs.copyTpl(
            this.templatePath('webapp/WEB-INF/jboss-deployment-structure.xml'),
            this.destinationPath(path + 'src/main/webapp/WEB-INF/jboss-deployment-structure.xml')
        );

        //creates jboss web config file
        this.fs.copyTpl(
            this.templatePath('webapp/WEB-INF/jboss-web.xml'),
            this.destinationPath(path + 'src/main/webapp/WEB-INF/jboss-web.xml'), {
                componentNameAllLowerDash: componentNameLowerDash,
            }
        );

        //creates web.xml file
        this.fs.copyTpl(
            this.templatePath('webapp/WEB-INF/web.xml'),
            this.destinationPath(path + 'src/main/webapp/WEB-INF/web.xml')
        );

        //test

        //java
        this.fs.copyTpl(
            this.templatePath('test/SpringContextTest.java'),
            this.destinationPath(path + 'src/test/java/' + rootPackagePath + '/SpringContextTest.java'), {
                rootPackageName: rootPackageName,
                componentNameCamel: componentNameCamel,
            }
        );

        this.fs.copyTpl(
            this.templatePath('test/Test.java'),
            this.destinationPath(path + 'src/test/java/' + rootPackagePath + '/annotation/' + componentNameCamel + 'Test.java'), {
                rootPackageName: rootPackageName,
                componentNameCamel: componentNameCamel,
            }
        );

        this.fs.copyTpl(
            this.templatePath('test/TestDataSourceConfig.java'),
            this.destinationPath(path + 'src/test/java/' + rootPackagePath + '/config/' + 'TestDataSourceConfig.java'), {
                rootPackageName: rootPackageName,
                componentNameCamel: componentNameCamel,
                componentNameCamelLower: componentNameCamelLower,
                componentNameLowerDash: componentNameLowerDash,
            }
        );

        //test resources

        this.fs.copyTpl(
            this.templatePath('test/resources/activiti.cfg.xml'),
            this.destinationPath(path + 'src/test/resources/activiti.cfg.xml')
        );

        this.fs.copyTpl(
            this.templatePath('test/resources/application-test.properties'),
            this.destinationPath(path + 'src/test/resources/application-test.properties'), {
                componentNameLowerDash: componentNameLowerDash
            }
        );

        this.fs.copyTpl(
            this.templatePath('test/resources/log4j2.xml'),
            this.destinationPath(path + 'src/test/resources/log4j2.xml')
        );

        //test resources process

        this.fs.copyTpl(
            this.templatePath('test/resources/process.xml'),
            this.destinationPath(path + 'src/test/resources/processes/process.xml')
        );

        //test resource docker

        this.fs.copyTpl(
            this.templatePath('test/resources/docker/Dockerfile'),
            this.destinationPath(path + 'src/test/resources/docker/postgresql-' + componentNameLowerDash + '/Dockerfile')
        );

        this.fs.copyTpl(
            this.templatePath('test/resources/docker/init-create-tablespaces.sh'),
            this.destinationPath(path + 'src/test/resources/docker/postgresql-' + componentNameLowerDash + '/init-create-tablespaces.sh')
        );
        this.fs.copyTpl(
            this.templatePath('test/resources/docker/init-postgres.sql'),
            this.destinationPath(path + 'src/test/resources/docker/postgresql-' + componentNameLowerDash + '/init-postgres.sql')
        );

        //test resource docker tsearch_data

        this.fs.copyTpl(
            this.templatePath('test/resources/docker/tsearch_data/czech.affix'),
            this.destinationPath(path + 'src/test/resources/docker/postgresql-' + componentNameLowerDash + '/tsearch_data/czech.affix')
        );

        this.fs.copyTpl(
            this.templatePath('test/resources/docker/tsearch_data/czech.dict'),
            this.destinationPath(path + 'src/test/resources/docker/postgresql-' + componentNameLowerDash + '/tsearch_data/czech.dict')
        );

        this.fs.copyTpl(
            this.templatePath('test/resources/docker/tsearch_data/czech.stop'),
            this.destinationPath(path + 'src/test/resources/docker/postgresql-' + componentNameLowerDash + '/tsearch_data/czech.stop')
        );

        //test resources changelog
        this.fs.copyTpl(
            this.templatePath('test/resources/changelog/db.changelog-parametr-init.xml'),
            this.destinationPath(path + 'src/test/resources/changelog/db.changelog-parametr-init.xml')
        );

        this.fs.copyTpl(
            this.templatePath('test/resources/changelog/db.changelog-test.xml'),
            this.destinationPath(path + 'src/test/resources/changelog/db.changelog-test.xml'), {
                componentNameLowerDash: componentNameLowerDash
            }
        );

        this.fs.copyTpl(
            this.templatePath('test/resources/changelog/dbchangelog-parametry.sql'),
            this.destinationPath(path + 'src/test/resources/changelog/dbchangelog-parametry.sql')
        );
    }

}
