package <%= rootPackageName %>.config.persistence;

import cz.cca.hedgehog.common.Constants;
import cz.cca.hedgehog.core.db.ExtendedSimpleJpaRepository;
import cz.cca.hedgehog.core.db.JndiPropertyHolder;
import liquibase.integration.spring.SpringLiquibase;
import net.ttddyy.dsproxy.listener.logging.SLF4JLogLevel;
import net.ttddyy.dsproxy.support.ProxyDataSourceBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.orm.jpa.EntityManagerFactoryBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.DependsOn;
import org.springframework.context.annotation.Primary;
import org.springframework.core.Ordered;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.datasource.lookup.JndiDataSourceLookup;
import org.springframework.lang.NonNull;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import org.springframework.transaction.support.DefaultTransactionStatus;
import org.springframework.util.Assert;

import javax.persistence.EntityManagerFactory;
import javax.sql.DataSource;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.function.BiConsumer;

/**
 * Konfigurační třída vytváří data source pro přístup k DB.
 */
@Configuration
@EnableTransactionManagement(order = Ordered.LOWEST_PRECEDENCE - 1)
@EnableJpaRepositories(
        entityManagerFactoryRef = "<%= componentNameLower %>EntityManagerFactory",
        transactionManagerRef = Constants.TRANSACTION_MANAGER,
        basePackages = {"<%= rootPackageName %>.db.repo", "cz.cca.hedgehog.user.db.repo"},
        repositoryBaseClass = ExtendedSimpleJpaRepository.class
)
public class <%= componentName %>DbConfig {

    @Primary
    @Bean(name = "<%= componentNameLower %>EntityManagerFactory")
    @DependsOn("<%= componentNameLower %>Liquibase")
    public LocalContainerEntityManagerFactoryBean <%= componentNameLower %>EntityManagerFactory(
            EntityManagerFactoryBuilder builder,
            @Qualifier("<%= componentNameLower %>DataSource") DataSource dataSource) {
        Map<String, Object> properties = new HashMap<>();
        properties.put("hibernate.hbm2ddl.auto", "validate");
        properties.put("hibernate.physical_naming_strategy", "<%= rootPackageName %>.config.persistence.<%= componentName %>PhysicalNamingStrategy");
        return builder
                .dataSource(dataSource)
                .persistenceUnit("<%= componentNameLower %>PersistenceUnit")
                .packages("<%= rootPackageName %>.db.domain",
                        "cz.cca.hedgehog.core.db.domain.konfigurace",
                        "cz.cca.hedgehog.user.db.domain")
                .properties(properties)
                .build();
    }

    @Bean
    @ConfigurationProperties(prefix = "spring.datasource")
    public JndiPropertyHolder <%= componentNameLower %>Jndi() {
        return new JndiPropertyHolder();
    }

    @Bean
    @ConfigurationProperties(prefix = "spring.liquibase-datasource")
    public JndiPropertyHolder <%= componentNameLower %>LiquibaseJndi() {
        return new JndiPropertyHolder();
    }

    @Primary
    @Bean(name = "<%= componentNameLower %>DataSource")
    public DataSource <%= componentNameLower %>DataSource() {
        return ProxyDataSourceBuilder
                .create(new JndiDataSourceLookup().getDataSource(<%= componentNameLower %>Jndi().getJndiName()))
                .name(Constants.DB_NAME)
                .logQueryBySlf4j(SLF4JLogLevel.DEBUG)
                .build();
    }

    @Bean("<%= componentNameLower %>LiquibaseDataSource")
    @ConditionalOnExpression("T(Boolean).parseBoolean('${<%= componentNameAllLowerDot %>.liquibase.enabled}')")
    public DataSource <%= componentNameLower %>LiquibaseDataSource() {
        return new JndiDataSourceLookup().getDataSource(<%= componentNameLower %>LiquibaseJndi().getJndiName());
    }

    @Bean
    @DependsOn("<%= componentNameLower %>DataSource")
    public SpringLiquibase <%= componentNameLower %>Liquibase(
            @Qualifier("<%= componentNameLower %>LiquibaseDataSource") @Autowired(required = false) DataSource dataSource,
            @Value("${spring.profiles.active:PROD}") String activeProfiles
    ) {
        boolean shouldRun = dataSource != null;
        SpringLiquibase springLiquibase = new SpringLiquibase();
        springLiquibase.setShouldRun(shouldRun);
        if (shouldRun) {
            springLiquibase.setDataSource(dataSource);
            springLiquibase.setContexts(activeProfiles);
        }
        springLiquibase.setChangeLog("classpath:changelog/<%= componentNameAllLowerDash %>-db.changelog-master.xml");
        return springLiquibase;
    }

    public static class JpaTransactionManagerWithSynchronizationPreparerCallback extends JpaTransactionManager {

        private final transient Map<Object, BiConsumer<DefaultTransactionStatus, TransactionDefinition>> synchronizationPreparerMap =
                Collections.synchronizedMap(new LinkedHashMap<>());

        public JpaTransactionManagerWithSynchronizationPreparerCallback() {
        }

        public JpaTransactionManagerWithSynchronizationPreparerCallback(EntityManagerFactory emf) {
            super(emf);
        }

        public void registerSynchronizationPreparer(
                Object key, BiConsumer<DefaultTransactionStatus, TransactionDefinition> synchronizationPreparer
        ) {
            Assert.notNull(synchronizationPreparer, "Synchronization preparer nesmí být null!");
            synchronizationPreparer = synchronizationPreparerMap.putIfAbsent(key, synchronizationPreparer);
            Assert.isNull(synchronizationPreparer, () -> String.format(
                    "Synchronization preparer s klíčem [%s] je již zaregistrován!", key
            ));
        }

        public void unregisterSynchronizationPreparer(Object key) {
            BiConsumer<DefaultTransactionStatus, TransactionDefinition> synchronizationPreparer =
                    synchronizationPreparerMap.remove(key);

            Assert.notNull(synchronizationPreparer, () -> String.format(
                    "Synchronization preparer s klíčem [%s] není registrován!", key
            ));
        }

        @Override
        protected void prepareSynchronization(
                @NonNull DefaultTransactionStatus transactionStatus, @NonNull TransactionDefinition transactionDefinition
        ) {
            super.prepareSynchronization(transactionStatus, transactionDefinition);
            for (BiConsumer<DefaultTransactionStatus, TransactionDefinition> preparer : synchronizationPreparerMap.values()) {
                preparer.accept(transactionStatus, transactionDefinition);
            }
        }

    }

    @Bean
    public JdbcTemplate jdbcTemplate(@Qualifier("<%= componentNameLower %>DataSource") DataSource dataSource) {
        return new JdbcTemplate(dataSource);
    }

    @Primary
    @Bean(name = Constants.TRANSACTION_MANAGER)
    public PlatformTransactionManager <%= componentNameLower %>TransactionManager(
            @Qualifier("<%= componentNameLower %>EntityManagerFactory") EntityManagerFactory entityManagerFactory
    ) {
        return new JpaTransactionManagerWithSynchronizationPreparerCallback(entityManagerFactory);
    }

}
