package  <%= rootPackageName %>.config;

import cz.cca.hedgehog.test.util.BeanDefinitionReplacerPostProcessor;
import cz.cca.hedgehog.test.util.TestDataSourceConfigUtils;
import cz.cca.hedgehog.test.util.TestDataSourceConfigUtils.TestDataSourceDefinition;
import liquibase.integration.spring.SpringLiquibase;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.support.BeanDefinitionRegistryPostProcessor;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.test.context.TestConfiguration;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.DependsOn;
import org.testcontainers.containers.GenericContainer;

import javax.sql.DataSource;

@TestConfiguration
public class TestDataSourceConfig {

    private final TestDataSourceConfigUtils utils = new TestDataSourceConfigUtils();

    /**
     * Zaregistruje {@link BeanDefinitionReplacerPostProcessor}, který zajistí nahrazení produkčních
     * data source bean definic za testovací.
     *
     * @implNote Tato metoda musí být {@code static}, protože bean-y implementující {@link
     * BeanDefinitionRegistryPostProcessor} jsou instancovány velmi brzy při startu application
     * contextu, tak brzy že pro ně ani nejsou vytvořeny proxy. Pokud by tato metoda nebyla {@code
     * static}, Spring by pro její zavolání musel vytvořit i instanci konfigurační třídy, ve které
     * je definovaná - v tomto případě {@link TestDataSourceConfigUtils}. T.j. instance této konfigurační
     * třídy by byla vytvořena také velmi brzy pří startu application contextu a nebyla by pro ni
     * vytvořena proxy. Důsledek toho by byl že přímá volání metod z této třídy anotovaných {@link
     * ConfigurationProperties &#64;ConfigurationProperties} by vracela prázdné objekty - objekty
     * jejichž všechny atributy jsou {@code null} - protože nastavení atributů těchto objektů
     * zajišťuje právě zmiňovaná proxy. To by mělo za následek nekorektní konfigurace testovacích
     * data source, při jejichž vytváření se tato přímá volání používají.
     */
    @Bean
    public static BeanDefinitionRegistryPostProcessor dataSourceOsobaReplacer() {
        return new BeanDefinitionReplacerPostProcessor(new String[][]{
                {"<%= componentNameCamelLower %>DataSource", "<%= componentNameCamelLower %>TestDataSource"}
        });
    }

    @Bean(initMethod = "start", destroyMethod = "stop")
    public GenericContainer postgresql<%= componentNameCamel %>Container() {
        return utils.postgresqlContainer("docker/postgresql-<%= componentNameLowerDash %>/", "15432:5432");
    }

    @Bean
    @DependsOn("postgresql<%= componentNameCamel %>Container")
    public SpringLiquibase <%= componentNameCamelLower %>Liquibase(@Autowired @Qualifier("<%= componentNameCamelLower %>DataSource") DataSource dataSource) {
        return utils.liquibase(dataSource);
    }

    @Bean
    @ConfigurationProperties(prefix = "<%= componentNameLowerDash %>-test-datasource")
    @SuppressWarnings("WeakerAccess")
    public TestDataSourceDefinition <%= componentNameCamelLower %>TestDataSourceDef() {
        return new TestDataSourceDefinition();
    }

    @Bean
    @DependsOn("postgresql<%= componentNameCamel %>Container")
    public DataSource <%= componentNameCamelLower %>TestDataSource() {
        return utils.testDataSource(<%= componentNameCamelLower %>TestDataSourceDef());
    }
}
