'use strict';

const Generator = require('yeoman-generator');
//contains util function form strings
const stringUtil = require('./string-utils');
let path;

module.exports = class extends Generator {

    constructor(args, opts) {
        // Calling the super constructor is important so our generator is correctly set up
        super(args, opts);
        this.argument("projectPath", {
            type: String,
            required: false,
            description: "Path to project folder",
            default: ""
        });

        path = stringUtil.path(opts.projectPath);
    }

        /**
         * reads user inputs
         */
        async prompting() {
            this.answers = await this.prompt([
                {
                    type: "input",
                    name: "clientName",
                    message: "Component name, for which client is generated, format (example-name)",
                },
                {
                    type: "input",
                    name: "rootPackageName",
                    message: "root package name",
                },
                {
                    type: "input",
                    name: "endpointUrl",
                    message: "component endpoint, format (endpoint/url/example)"
                }
            ]);
        }

        /**
         * generete files from templates
         */
        async writing() {
            //declaration of all constants used in templates
            //package name seperated with dots
            const rootPackageName = this.answers.rootPackageName;
            //package name seperated with slashes
            const rootPackagePath = stringUtil.packagePath(rootPackageName);
            //package name with cut last package
            const rootPackageCut = stringUtil.cutLastPackage(rootPackageName);
            //client name seperated with dashes all lower case
            const clientNameAllLower =  this.answers.clientName.toLocaleLowerCase();
            //client name seperated with spaces
            const clientNameSpace = stringUtil.spaceFirstLettersUpperCase(clientNameAllLower);
            //client name seperated with spaces all lower
            const clientNameAllLowerSpace = clientNameSpace.toLocaleLowerCase();
            //client name in camel case first letter Upper case
            const clientNameCamel = stringUtil.camelCase(clientNameAllLower);
            //client name words seperated with dots
            const clientNameDot = stringUtil.dotSeperator(clientNameAllLower);
            //endpoint of component for which client is made
            const endpointUrl = stringUtil.path(this.answers.endpointUrl);

            //project root
            this.fs.copyTpl(
                this.templatePath('pom.xml'),
                this.destinationPath(path + 'pom.xml'), {
                    clientNameDash: clientNameAllLower,
                    clientNameSpace: clientNameSpace,
                    clientNameAllLowerSpace: clientNameAllLowerSpace,
                    packageNameCut: rootPackageCut,
                }
            );

            this.fs.copyTpl(
                this.templatePath('.gitignore'),
                this.destinationPath(path + '.gitignore')
            );

            //java classes
            this.fs.copyTpl(
                this.templatePath('java/Service.java'),
                this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/klient/service/' + clientNameCamel + 'Service.java'), {
                    rootPackageName: rootPackageName,
                    clientName: clientNameCamel,
                }
            );
            this.fs.copyTpl(
                this.templatePath('java/ServiceImpl.java'),
                this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/klient/service/' + clientNameCamel + 'ServiceImpl.java'), {
                    rootPackageName: rootPackageName,
                    clientName: clientNameCamel,
                    clientNameDot: clientNameDot,
                    rootPackageCut: rootPackageCut,
                    endpointUrl: endpointUrl,
                }
            );
        }
}
